#include "pipe_paging.h"

PipePaging::PipePaging(offset_t size)
: _size(size)
{
}

/* Detach one endpoint. */

void PipePaging::detach()
{
    if (!_endpoints)
        return;
    else
        _endpoints--;

    /* Return if the other endpoint is attached. */

    if (_endpoints)
        return;

    /* Discard all regions from the pipe. */

    while (!_regions.empty())
    {
        PageMapper *mapper = _regions.front();

        _regions.pop_front();
        mapper->detach();
        delete mapper;
    }
}

/* Add a region to the sequence. */

void PipePaging::add_region(PageMapper *mapper)
{
    _regions.push_back(mapper);
}

/* Return the first region in the sequence. */

PageMapper *PipePaging::first_region()
{
    return _regions.front();
}

/* Return the next region for the reader. If only a single region remains, with
   the reader wishing to move to the next, return NULL. */

PageMapper *PipePaging::next_region()
{
    if (_regions.size() > 1)
    {
        /* Discard the accessor from the current region. */

        if (_accessors.size() > _regions.size())
            _accessors.pop_front();

        /* Detach and discard the current page mapper. */

        PageMapper *mapper = _regions.front();

        _regions.pop_front();
        mapper->detach();
        delete mapper;

        /* Return the next region. */

        return _regions.front();
    }
    else
        return NULL;
}

/* Initialise an accessor for a region. */

PipeAccessor *PipePaging::accessor()
{
    PipeAccessor accessor;

    _accessors.push_back(accessor);
    return &_accessors.back();
}

// vim: tabstop=4 expandtab shiftwidth=4
